import scrollIntoView from 'scroll-into-view-if-needed';
var memoizedNow;

var now = function now() {
  if (!memoizedNow) {
    memoizedNow = 'performance' in window ? performance.now.bind(performance) : Date.now;
  }

  return memoizedNow();
};

function step(context) {
  var time = now();
  var elapsed = Math.min((time - context.startTime) / context.duration, 1);
  var value = context.ease(elapsed);
  var currentX = context.startX + (context.x - context.startX) * value;
  var currentY = context.startY + (context.y - context.startY) * value;
  context.method(currentX, currentY);

  if (currentX !== context.x || currentY !== context.y) {
    requestAnimationFrame(function () {
      return step(context);
    });
  }
}

function smoothScroll(el, x, y, duration, ease, cb) {
  if (duration === void 0) {
    duration = 300;
  }

  if (ease === void 0) {
    ease = function ease(t) {
      return 0.5 * (1 - Math.cos(Math.PI * t));
    };
  }

  var scrollable;
  var startX;
  var startY;
  var method;

  if (el === document.documentElement) {
    scrollable = window;
    startX = window.scrollX || window.pageXOffset;
    startY = window.scrollY || window.pageYOffset;

    method = function method(x, y) {
      return window.scroll(x, y);
    };
  } else {
    scrollable = el;
    startX = el.scrollLeft;
    startY = el.scrollTop;

    method = function method(x, y) {
      el.scrollLeft = x;
      el.scrollTop = y;
    };
  }

  step({
    scrollable: scrollable,
    method: method,
    startTime: now(),
    startX: startX,
    startY: startY,
    x: x,
    y: y,
    duration: duration,
    ease: ease,
    cb: cb
  });
}

var shouldSmoothScroll = function shouldSmoothScroll(options) {
  return options && !options.behavior || options.behavior === 'smooth';
};

function scroll(target, options) {
  if (shouldSmoothScroll(options)) {
    var overrides = options || {};
    return scrollIntoView(target, {
      block: overrides.block,
      inline: overrides.inline,
      scrollMode: overrides.scrollMode,
      boundary: overrides.boundary,
      behavior: function behavior(actions) {
        return Promise.all(actions.map(function (_ref) {
          var el = _ref.el,
              left = _ref.left,
              top = _ref.top;
          return new Promise(function (resolve) {
            return smoothScroll(el, left, top, overrides.duration, overrides.ease, function () {
              return resolve();
            });
          });
        }));
      }
    });
  }

  return scrollIntoView(target, options);
}

var smoothScrollIntoView = scroll;
export default smoothScrollIntoView;